<?php
class Bazaarvoice_Connector_Block_Roi_Beacon extends Mage_Core_Block_Template
{
    private $_isEnabled;

    public function _construct()
    {
        // enabled/disabled in admin
        $this->_isEnabled = Mage::getStoreConfig('bazaarvoice/general/enable_roibeacon') === '1'
                                && Mage::getStoreConfig('bazaarvoice/general/enable_bv') === '1';
    }

    /**
     * returns true if feature is enabled in admin, otherwise returns false
     * @return bool
     */
    public function getIsEnabled()
    {
        return $this->_isEnabled;
    }

    /**
     * returns serialized order details data for transmission to Bazaarvoice
     * @return string
     */
    public function getOrderDetails()
    {
        $orderDetails = array();
        $result = '';
        $orderId = Mage::getSingleton('checkout/session')->getLastOrderId();
        if ($orderId)
        {
            $order = Mage::getModel('sales/order')->load($orderId);
            if ($order->getId())
            {
                $address = $order->getBillingAddress();

                $orderDetails['orderId'] = $order->getIncrementId();

                $total = $order->getGrandTotal() - $order->getTaxAmount() - $order->getShippingAmount();
                $orderDetails['total'] = number_format($total, 2, '.', '');
                $orderDetails['tax'] = number_format($order->getTaxAmount(), 2, '.', '');
                $orderDetails['shipping'] = number_format($order->getShippingAmount(), 2, '.', '');

                $orderDetails['city'] = $address->getCity();
                $orderDetails['state'] = Mage::getModel('directory/region')->load($address->getRegionId())->getCode();
                $orderDetails['country'] = $address->getCountryId();
                $orderDetails['currency'] = $order->getOrderCurrencyCode();

                $orderDetails['items'] = array();
                // if families are enabled, get all items
                if(Mage::getStoreConfig('bazaarvoice/feeds/families')){
                    $items = $order->getAllItems();
                } else {
                    $items = $order->getAllVisibleItems();
                }
                foreach ($items as $itemId => $item)
                {
                    $product = Mage::helper('bazaarvoice')->getReviewableProductFromOrderItem($item);
                    $product = Mage::getModel('catalog/product')->load($product->getId());
                    // skip configurable items if families are enabled
                    if(Mage::getStoreConfig('bazaarvoice/feeds/families') && $product->getTypeId() == Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE) continue;
                     
                    $itemDetails = array();
                    $itemDetails['sku'] = Mage::helper('bazaarvoice')->getProductId($product);
                    $itemDetails['name'] = $item->getName();
                    // 'category' is not included.  Mage products can be in 0 - many categories.  Should we try to include it?
                    $itemDetails['price'] = number_format($item->getPrice(), 2, '.', '');
                    $itemDetails['quantity'] = number_format($item->getQtyOrdered(), 0);
                    $itemDetails['imageURL'] = $product->getImageUrl();
                    
                    if(Mage::getStoreConfig('bazaarvoice/feeds/families') && $item->getParentItem()) {
                        if(strpos($itemDetails['imageURL'], "placeholder/image.jpg")) {
                            // if product families are enabled and product has no image, use configurable image
                            $parentId = $item->getParentItem()->getProductId();
                            $parent = Mage::getModel('catalog/product')->load($parentId);
                            $itemDetails['imageURL'] = $parent->getImageUrl();
                        }
                        // also get price from parent item
                        $itemDetails['price'] = number_format($item->getParentItem()->getPrice(), 2, '.', '');
                    }
                    
                    array_push($orderDetails['items'], $itemDetails);
                }
                
                if($order->getCustomerId()) {
                    $userId = $order->getCustomerId();
                } else {
                    $userId = md5($order->getCustomerEmail());
                }
                $orderDetails['userId'] = $userId;
                $orderDetails['email'] = $order->getCustomerEmail();
                $orderDetails['nickname'] = $order->getCustomerFirstname();
                // There is no 'deliveryDate' yet
                $orderDetails['locale'] = Mage::getStoreConfig('bazaarvoice/general/locale', $order->getStoreId());

                // Add partnerSource field
                $orderDetails['partnerSource'] = 'Magento Extension r' . Mage::helper('bazaarvoice')->getExtensionVersion();
                $deploymentZone = Mage::getStoreConfig('bazaarvoice/general/deployment_zone', $order->getStoreId());
                $deploymentZone = strtolower(str_replace(' ', '_', $deploymentZone));
                $orderDetails['deploymentZone'] = $deploymentZone;

                $loader = '<script src="//apps.bazaarvoice.com/deployments/'
                    . Mage::getStoreConfig('bazaarvoice/general/client_name', $order->getStoreId())
                    . '/' . $deploymentZone
                    . '/' . Mage::getStoreConfig('bazaarvoice/general/environment', $order->getStoreId())
                    . '/' . Mage::getStoreConfig('bazaarvoice/general/locale', $order->getStoreId())
                    . '/bv.js"></script>';

                Mage::log($orderDetails, Zend_Log::DEBUG, Bazaarvoice_Connector_Helper_Data::LOG_FILE);
                $result = '';
                if(Mage::getStoreConfig('bazaarvoice/general/environment') == "staging") {
                    $result = '
                    <!--
                    ' . print_r($orderDetails, 1) . '
                    -->';
                }
                $result .= $loader."\n";
                $result .= '
                <script type="text/javascript">
                    var transactionData = ' . json_encode($orderDetails, JSON_UNESCAPED_UNICODE) . ';
                    BV.pixel.trackTransaction(transactionData);
                </script>';
            }
        }
        return $result;
    }
}
